'' =================================================================================================
''
''   File....... jm_launch_code_v2.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2010 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 19 MAY 2010
''
'' =================================================================================================


CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
' _xinfreq = 6_250_000   

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000


CON 

  #1, HOME, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR, #16, CLS      ' PST formmatting control


CON

  BTN_UP   = %000001                                            ' for LCD UI board 
  BTN_DN   = %000010 
  BTN_LF   = %000100 
  BTN_RT   = %001000 
  BTN_OK   = %010000 
  BTN_ESC  = %100000 

  BTN_MASK = BTN_UP | BTN_DN | BTN_RT | BTN_LF | BTN_OK | BTN_ESC

  
OBJ

  term : "fullduplexserial"
  lcd  : "jm_lcd4"
  btns : "jm_debounce"   


VAR

  byte  launchcode[6]                                           ' 6 + zero (for str functions)


PUB main | t0, pos, idx, scan, blink, ofs
                                          
  term.start(31, 30, %0000, 115_200)
  lcd.init(8, 8, 2)
  btns.init(BTN_MASK << 18, 25, 1)

  bytemove(@launchcode, @DefaultCode, 7)

  pause(1)
  term.tx(CLS)
     
  lcd.cmd(lcd#CLS)
  lcd.str(@launchcode)

  pos~
  blink~

  t0 := cnt
  
  repeat    
    repeat idx from 0 to 5
      if (idx == pos) and (blink == 1)
        ofs := (launchcode[pos] - "0") << 3                     ' offset of inverted digit
        lcd.setchar(0, @InvDigit + ofs)                         ' write to custom char 0
        lcd.moveto(idx + 1, 1)                                  ' repostion in DDRAM
        lcd.out(0)
      else
        lcd.moveto(idx + 1, 1)
        lcd.out(launchcode[idx])

    scan := btns.getgroup(23, 18)
    case scan
      BTN_UP : launchcode[pos] := nextdigit(launchcode[pos])                     
      BTN_DN : launchcode[pos] := prevdigit(launchcode[pos])  
      BTN_RT : pos := ++pos // 6
      BTN_LF : pos := (pos + 5) // 6 

    blink := 1 - blink
    waitcnt(t0 += clkfreq >> 2)                                 ' for fixed loop timing


PUB nextalpha(alpha)

  if alpha < "Z"
    ++alpha
  else
    alpha := "A"

  return alpha


PUB prevalpha(alpha)

  if alpha > "A"
    --alpha
  else
    alpha := "Z"

  return alpha


PUB nextdigit(digit)

  if digit < "9"
    ++digit
  else
    digit := "0"

  return digit


PUB prevdigit(digit)

  if digit > "0"
    --digit
  else
    digit := "9"

  return digit
      
 
PUB pause(ms) | t

  t := cnt - 1088                                               ' sync with system counter
  repeat ms
    waitcnt(t += MS_001)


DAT

DefaultCode             byte    "000000", 0


DAT

InvDigit                byte    $11, $0E, $0C, $0A, $06, $0E, $11, $1F          ' 0
                        byte    $1B, $13, $1B, $1B, $1B, $1B, $11, $1F
                        byte    $11, $0E, $1E, $1D, $1B, $17, $00, $1F
                        byte    $00, $1D, $1B, $1D, $1E, $0E, $11, $1F
                        byte    $1D, $19, $15, $0D, $00, $1D, $1D, $1F
                        byte    $00, $0F, $01, $1E, $1E, $0E, $11, $1F
                        byte    $19, $17, $0F, $01, $0E, $0E, $11, $1F
                        byte    $00, $1E, $1D, $1B, $17, $17, $17, $1F
                        byte    $11, $0E, $0E, $11, $0E, $0E, $11, $1F
                        byte    $11, $0E, $0E, $10, $1E, $1D, $13, $1F          ' 9

Diamond                 byte    $00, $04, $0E, $1F, $0E, $04, $00, $00
                        
                        
DAT

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  